@@ -71,12 +71,8 @@ scrollBox = propertyList "scroll server" $ props
 		, "fi"
 		, "echo Your game was recorded, as ID:$(basename \"$t\")"
 		, "echo if you would like to talk about how it went, email scroll@joeyh.name"
-		, "echo 'or, type comments below (finish with a dot on its own line)'"
-		, "echo"
-		, "echo Your comments:"
-		, "timeout --kill-after 1m --foreground 2h mail -E -s \"scroll test $t\" joey@kitenet.net"
+		, "read line"
 		] `onChange` (g `File.mode` (combineModes (ownerWriteMode:readModes ++ executeModes)))
-	& Apt.installed ["bsd-mailx"]
 	-- prevent port forwarding etc by not letting scroll log in via ssh
 	& Ssh.sshdConfig `File.containsLine` ("DenyUsers scroll")
 		`onChange` Ssh.restarted
@@ -362,40 +358,17 @@ downloads hosts = annexWebSite "/srv/git/downloads.git"
 	`requires` Ssh.knownHost hosts "eubackup.kitenet.net" (User "joey")
 
 tmp :: Property (HasInfo + DebianLike)
-tmp = propertyList "tmp.kitenet.net" $ props
+tmp = propertyList "tmp.joeyh.name" $ props
 	& annexWebSite "/srv/git/joey/tmp.git"
-		"tmp.kitenet.net"
+		"tmp.joeyh.name"
 		"26fd6e38-1226-11e2-a75f-ff007033bdba"
 		[]
-	& twitRss
 	& pumpRss
 
--- Twitter, you kill us.
-twitRss :: Property DebianLike
-twitRss = combineProperties "twitter rss" $ props
-	& Git.cloned (User "joey") "git://git.kitenet.net/twitrss.git" dir Nothing
-	& check (not <$> doesFileExist (dir </> "twitRss")) compiled
-	& feed "http://twitter.com/search/realtime?q=git-annex" "git-annex-twitter"
-	& feed "http://twitter.com/search/realtime?q=olduse+OR+git-annex+OR+debhelper+OR+etckeeper+OR+ikiwiki+-ashley_ikiwiki" "twittergrep"
-  where
-	dir = "/srv/web/tmp.kitenet.net/twitrss"
-	crontime = Cron.Times "15 * * * *"
-	feed url desc = Cron.job desc crontime (User "joey") dir $
-		"./twitRss " ++ shellEscape url ++ " > " ++ shellEscape ("../" ++ desc ++ ".rss")
-	compiled = userScriptProperty (User "joey")
-		[ "cd " ++ dir
-		, "ghc --make twitRss"
-		]
-		`assume` NoChange
-		`requires` Apt.installed
-			[ "libghc-xml-dev"
-			, "libghc-feed-dev"
-			, "libghc-tagsoup-dev"
-			]
-
 -- Work around for expired ssl cert.
+-- (Obsolete; need to revert this.)
 pumpRss :: Property DebianLike
-pumpRss = Cron.job "pump rss" (Cron.Times "15 * * * *") (User "joey") "/srv/web/tmp.kitenet.net/"
+pumpRss = Cron.job "pump rss" (Cron.Times "15 * * * *") (User "joey") "/srv/web/tmp.joeyh.name/"
 	"wget https://rss.io.jpope.org/feed/joeyh@identi.ca.atom -O pump.atom.new --no-check-certificate 2>/dev/null; sed 's/ & / /g' pump.atom.new > pump.atom"
 
 ircBouncer :: Property (HasInfo + DebianLike)
@@ -540,6 +513,7 @@ kiteMailServer = propertyList "kitenet.net mail server" $ props
 	& Fail2Ban.jailEnabled "postfix-sasl"
 	& "/etc/default/saslauthd" `File.containsLine` "MECHANISMS=sasldb"
 	& Postfix.saslPasswdSet "kitenet.net" (User "errol")
+	& Postfix.saslPasswdSet "kitenet.net" (User "joey")
 
 	& Apt.installed ["maildrop"]
 	& "/etc/maildroprc" `File.hasContent`
@@ -558,7 +532,6 @@ kiteMailServer = propertyList "kitenet.net mail server" $ props
 
 	& "/etc/aliases" `File.hasPrivContentExposed` ctx
 		`onChange` Postfix.newaliases
-	& hasJoeyCAChain
 	& hasPostfixCert ctx
 
 	& "/etc/postfix/mydomain" `File.containsLines`
@@ -605,7 +578,7 @@ kiteMailServer = propertyList "kitenet.net mail server" $ props
 		, "# Filter out client relay lines from headers."
 		, "header_checks = pcre:$config_directory/obscure_client_relay.pcre"
 
-		, "# Password auth for relaying (used by errol)"
+		, "# Password auth for relaying"
 		, "smtpd_sasl_auth_enable = yes"
 		, "smtpd_sasl_security_options = noanonymous"
 		, "smtpd_sasl_local_domain = kitenet.net"
@@ -681,6 +654,9 @@ kiteMailServer = propertyList "kitenet.net mail server" $ props
 		`describe` "pine configured to use local imap server"
 
 	& Apt.serviceInstalledRunning "mailman"
+	-- Override the default http url. (Only affects new lists.)
+	& "/etc/mailman/mm_cfg.py" `File.containsLine`
+		"DEFAULT_URL_PATTERN = 'https://%s/cgi-bin/mailman/'"
 
 	& Postfix.service ssmtp
 
@@ -694,24 +670,6 @@ kiteMailServer = propertyList "kitenet.net mail server" $ props
 		(Postfix.InetService Nothing "ssmtp")
 		"smtpd" Postfix.defServiceOpts
 
--- Configures postfix to relay outgoing mail to kitenet.net, with
--- verification via tls cert.
-postfixClientRelay :: Context -> Property (HasInfo + DebianLike)
-postfixClientRelay ctx = Postfix.mainCfFile `File.containsLines`
-	-- Using smtps not smtp because more networks firewall smtp
-	[ "relayhost = kitenet.net:smtps"
-	, "smtp_tls_CAfile = /etc/ssl/certs/joeyca.pem"
-	, "smtp_tls_cert_file = /etc/ssl/certs/postfix.pem"
-	, "smtp_tls_key_file = /etc/ssl/private/postfix.pem"
-	, "smtp_tls_loglevel = 0"
-	, "smtp_use_tls = yes"
-	]
-	`describe` "postfix client relay"
-	`onChange` Postfix.dedupMainCf
-	`onChange` Postfix.reloaded
-	`requires` hasJoeyCAChain
-	`requires` hasPostfixCert ctx
-
 -- Configures postfix to have the dkim milter, and no other milters.
 dkimMilter :: Property (HasInfo + DebianLike)
 dkimMilter = Postfix.mainCfFile `File.containsLines`
@@ -767,7 +725,73 @@ legacyWebSites = propertyList "legacy web sites" $ props
 	& Apache.modEnabled "cgi"
 	& Apache.modEnabled "speling"
 	& userDirHtml
-	& Apache.httpsVirtualHost' "kitenet.net" "/var/www" letos
+	& Apache.httpsVirtualHost' "kitenet.net" "/var/www" letos kitenetcfg
+	& alias "anna.kitenet.net"
+	& apacheSite "anna.kitenet.net"
+		[ "DocumentRoot /home/anna/html"
+		, "<Directory /home/anna/html/>"
+		, "  Options Indexes ExecCGI"
+		, "  AllowOverride None"
+		, Apache.allowAll
+		, "</Directory>"
+		]
+	& alias "sows-ear.kitenet.net"
+	& alias "www.sows-ear.kitenet.net"
+	& apacheSite "sows-ear.kitenet.net"
+		[ "ServerAlias www.sows-ear.kitenet.net"
+		, "DocumentRoot /srv/web/sows-ear.kitenet.net"
+		, "<Directory /srv/web/sows-ear.kitenet.net>"
+		, "  Options FollowSymLinks"
+		, "  AllowOverride None"
+		, Apache.allowAll
+		, "</Directory>"
+		, "RewriteEngine On"
+		, "RewriteRule .* http://www.sowsearpoetry.org/ [L]"
+		]
+	& alias "wortroot.kitenet.net"
+	& alias "www.wortroot.kitenet.net"
+	& apacheSite "wortroot.kitenet.net"
+		[ "ServerAlias www.wortroot.kitenet.net"
+		, "DocumentRoot /srv/web/wortroot.kitenet.net"
+		, "<Directory /srv/web/wortroot.kitenet.net>"
+		, "  Options FollowSymLinks"
+		, "  AllowOverride None"
+		, Apache.allowAll
+		, "</Directory>"
+		]
+	& alias "creeksidepress.com"
+	& apacheSite "creeksidepress.com"
+		[ "ServerAlias www.creeksidepress.com"
+		, "DocumentRoot /srv/web/www.creeksidepress.com"
+		, "<Directory /srv/web/www.creeksidepress.com>"
+		, "  Options FollowSymLinks"
+		, "  AllowOverride None"
+		, Apache.allowAll
+		, "</Directory>"
+		]
+	& alias "joey.kitenet.net"
+	& apacheSite "joey.kitenet.net"
+		[ "DocumentRoot /var/www"
+		, "<Directory /var/www/>"
+		, "  Options Indexes ExecCGI"
+		, "  AllowOverride None"
+		, Apache.allowAll
+		, "</Directory>"
+
+		, "RewriteEngine On"
+
+		, "# Old ikiwiki filenames for joey's wiki."
+		, "rewritecond $1 !.*/index$"
+		, "rewriterule (.+).html$ http://joeyh.name/$1/ [l]"
+
+		, "rewritecond $1 !.*/index$"
+		, "rewriterule (.+).rss$ http://joeyh.name/$1/index.rss [l]"
+
+		, "# Redirect all to joeyh.name."
+		, "rewriterule (.*) http://joeyh.name$1 [r]"
+		]
+  where
+	kitenetcfg =
 		-- /var/www is empty
 		[ "DocumentRoot /var/www"
 		, "<Directory /var/www>"
@@ -854,70 +878,6 @@ legacyWebSites = propertyList "legacy web sites" $ props
 		, "rewriterule /~kyle/family/wiki/(.*).rss http://macleawiki.branchable.com/$1/index.rss [L]"
 		, "rewriterule /~kyle/family/wiki(.*) http://macleawiki.branchable.com$1 [L]"
 		]
-	& alias "anna.kitenet.net"
-	& apacheSite "anna.kitenet.net"
-		[ "DocumentRoot /home/anna/html"
-		, "<Directory /home/anna/html/>"
-		, "  Options Indexes ExecCGI"
-		, "  AllowOverride None"
-		, Apache.allowAll
-		, "</Directory>"
-		]
-	& alias "sows-ear.kitenet.net"
-	& alias "www.sows-ear.kitenet.net"
-	& apacheSite "sows-ear.kitenet.net"
-		[ "ServerAlias www.sows-ear.kitenet.net"
-		, "DocumentRoot /srv/web/sows-ear.kitenet.net"
-		, "<Directory /srv/web/sows-ear.kitenet.net>"
-		, "  Options FollowSymLinks"
-		, "  AllowOverride None"
-		, Apache.allowAll
-		, "</Directory>"
-		, "RewriteEngine On"
-		, "RewriteRule .* http://www.sowsearpoetry.org/ [L]"
-		]
-	& alias "wortroot.kitenet.net"
-	& alias "www.wortroot.kitenet.net"
-	& apacheSite "wortroot.kitenet.net"
-		[ "ServerAlias www.wortroot.kitenet.net"
-		, "DocumentRoot /srv/web/wortroot.kitenet.net"
-		, "<Directory /srv/web/wortroot.kitenet.net>"
-		, "  Options FollowSymLinks"
-		, "  AllowOverride None"
-		, Apache.allowAll
-		, "</Directory>"
-		]
-	& alias "creeksidepress.com"
-	& apacheSite "creeksidepress.com"
-		[ "ServerAlias www.creeksidepress.com"
-		, "DocumentRoot /srv/web/www.creeksidepress.com"
-		, "<Directory /srv/web/www.creeksidepress.com>"
-		, "  Options FollowSymLinks"
-		, "  AllowOverride None"
-		, Apache.allowAll
-		, "</Directory>"
-		]
-	& alias "joey.kitenet.net"
-	& apacheSite "joey.kitenet.net"
-		[ "DocumentRoot /var/www"
-		, "<Directory /var/www/>"
-		, "  Options Indexes ExecCGI"
-		, "  AllowOverride None"
-		, Apache.allowAll
-		, "</Directory>"
-
-		, "RewriteEngine On"
-
-		, "# Old ikiwiki filenames for joey's wiki."
-		, "rewritecond $1 !.*/index$"
-		, "rewriterule (.+).html$ http://joeyh.name/$1/ [l]"
-
-		, "rewritecond $1 !.*/index$"
-		, "rewriterule (.+).rss$ http://joeyh.name/$1/index.rss [l]"
-
-		, "# Redirect all to joeyh.name."
-		, "rewriterule (.*) http://joeyh.name$1 [r]"
-		]
 
 userDirHtml :: Property DebianLike
 userDirHtml = File.fileProperty "apache userdir is html" (map munge) conf
